@@ -21,7 +21,8 @@
 */
 
 #include <config.h>
-#include <gal/widgets/e-unicode.h>
+#include <libgnome/gnome-i18n.h>
+#include <string.h>
 #include <gal/widgets/widget-color-combo.h>
 #include "htmlengine-edit.h"
 #include "htmlengine-edit-clueflowstyle.h"
@@ -92,15 +93,17 @@ static BodyTemplate body_templates [TEMPLATES] = {
 		{0, 0, 0, 0},
 		{0, 0, 0, 0xffff},
 		10,
-	},
+  	},
 };
 
 static void
 fill_sample (GtkHTMLEditBodyProperties *d)
 {
-	gchar *body, *body_tag, *bg_image, *fname, *lm;
-	fname = gtk_entry_get_text (GTK_ENTRY (gnome_pixmap_entry_gtk_entry
-					       (GNOME_PIXMAP_ENTRY (d->pixmap_entry))));
+	gchar *body, *body_tag, *bg_image, *lm;
+	const gchar *fname;
+
+	fname = gtk_entry_get_text (GTK_ENTRY (gnome_file_entry_gtk_entry
+					       (GNOME_FILE_ENTRY (d->pixmap_entry))));
 	bg_image = fname && *fname ? g_strdup_printf (" BACKGROUND=\"%s\"", fname) : g_strdup ("");
 	lm = d->left_margin != 10 ? g_strdup_printf (" LEFTMARGIN=%d", d->left_margin) : g_strdup ("");
 	body_tag = g_strdup_printf ("<BODY BGCOLOR=#%02x%02x%02x LINK=#%02x%02x%02x TEXT=#%02x%02x%02x%s%s>",
@@ -116,7 +119,8 @@ fill_sample (GtkHTMLEditBodyProperties *d)
 				    bg_image, lm);
 
 	body  = g_strconcat (body_tag,
-			     "The quick brown <a href=\"mailto:fox\">fox</a> jumps over the lazy <a href=\"mailto:dog\">dog</a>.",
+			     /* string marked for translations is sample text. you may want to translate it to common sample text in your language */
+			     _("The quick brown <a href=\"mailto:fox\">fox</a> jumps over the lazy <a href=\"mailto:dog\">dog</a>."),
 			     NULL);
 
 	gtk_html_load_from_string (d->sample, body, -1);
@@ -127,7 +131,8 @@ fill_sample (GtkHTMLEditBodyProperties *d)
 }
 
 static void
-color_changed (GtkWidget *w, GdkColor *color, gboolean by_user, GtkHTMLEditBodyProperties *data)
+color_changed (GtkWidget *w, GdkColor *color, gboolean custom, gboolean by_user, gboolean is_default,
+	       GtkHTMLEditBodyProperties *data)
 {
 	gint idx;
 
@@ -135,7 +140,7 @@ color_changed (GtkWidget *w, GdkColor *color, gboolean by_user, GtkHTMLEditBodyP
 	if (!by_user)
 		return;
 		
-	idx = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (w), "type"));
+	idx = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (w), "type"));
 	data->color [idx] = color
 		? *color
 		: html_colorset_get_color (data->cd->html->engine->defaultSettings->color_set, idx)->color;
@@ -164,7 +169,7 @@ changed_template (GtkWidget *w, GtkHTMLEditBodyProperties *d)
 {
 	d->template = g_list_index (GTK_MENU_SHELL (w)->children, gtk_menu_get_active (GTK_MENU (w)));
 
-	gtk_entry_set_text (GTK_ENTRY (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->pixmap_entry))),
+	gtk_entry_set_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (d->pixmap_entry))),
 			    body_templates [d->template].bg_pixmap ? body_templates [d->template].bg_pixmap : "");
 
 	if (d->template) {
@@ -220,11 +225,11 @@ fill_templates (GtkHTMLEditBodyProperties *d)
 		GtkWidget *item;
 
 		item = gtk_menu_item_new_with_label (_(body_templates [i].name));
-		gtk_menu_append (GTK_MENU (menu), item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
-		gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &body_templates [i].bg_color);
-		gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &body_templates [i].text_color);
-		gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &body_templates [i].link_color);
+		/* RM2 gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &body_templates [i].bg_color);
+		   gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &body_templates [i].text_color);
+		   gdk_color_alloc (gdk_window_get_colormap (GTK_WIDGET (d->cd->html)->window), &body_templates [i].link_color); */
 	}
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (d->option_template), menu);
 }
@@ -243,43 +248,39 @@ body_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	data->left_margin = 10;
 
 	table = gtk_table_new (2, 2, FALSE);
-	gtk_container_border_width (GTK_CONTAINER (table), 3);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 3);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 12);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
 
-	vb1 = gtk_vbox_new (FALSE, 0);
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
+	vb1 = gtk_vbox_new (FALSE, 6);
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 	frame = gtk_frame_new (_("Template"));
 	data->option_template = gtk_option_menu_new ();
 	fill_templates (data);
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (data->option_template))),
-			    "selection-done", changed_template, data);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (data->option_template)),
+			  "selection-done", G_CALLBACK (changed_template), data);
 	gtk_box_pack_start (GTK_BOX (hbox), data->option_template, FALSE, TRUE, 0);
 	gtk_container_add (GTK_CONTAINER (frame), hbox);
 	gtk_box_pack_start (GTK_BOX (vb1), frame, FALSE, TRUE, 0);
 	
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 	data->entry_title = gtk_entry_new ();
 	if (gtk_html_get_title (data->cd->html)) {
-		e_utf8_gtk_entry_set_text (GTK_ENTRY (data->entry_title), 
-					   gtk_html_get_title (data->cd->html));
+		gtk_entry_set_text (GTK_ENTRY (data->entry_title), 
+				    gtk_html_get_title (data->cd->html));
 	}
-	gtk_signal_connect (GTK_OBJECT (data->entry_title), "changed", entry_changed, data);
+	g_signal_connect (data->entry_title, "changed", G_CALLBACK (entry_changed), data);
 	gtk_box_pack_start_defaults (GTK_BOX (hbox), data->entry_title);
 	frame = gtk_frame_new (_("Document Title"));
 	gtk_container_add (GTK_CONTAINER (frame), hbox);
 	gtk_box_pack_start (GTK_BOX (vb1), frame, FALSE, TRUE, 0);	
 
 	frame = gtk_frame_new (_("Background Image"));
-	vbox = gtk_vbox_new (FALSE, 2);
-	gtk_container_border_width (GTK_CONTAINER (vbox), 3);
-	data->pixmap_entry = gnome_pixmap_entry_new ("background_image", _("Background Image"), TRUE);
-
-	/* fix for broken gnome-libs, could be removed once gnome-libs are fixed */
-	gnome_entry_load_history (GNOME_ENTRY (gnome_pixmap_entry_gnome_entry (GNOME_PIXMAP_ENTRY (data->pixmap_entry))));
-	our_gnome_pixmap_entry_set_last_dir (GNOME_PIXMAP_ENTRY (data->pixmap_entry));
+	vbox = gtk_vbox_new (FALSE, 4);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+	data->pixmap_entry = gnome_pixmap_entry_new ("background_image", _("Background Image"), FALSE);
 
 	if (cd->html->engine->bgPixmapPtr) {
 		HTMLImagePointer *ip = (HTMLImagePointer *) cd->html->engine->bgPixmapPtr;
@@ -287,12 +288,12 @@ body_properties (GtkHTMLControlData *cd, gpointer *set_data)
 		 if (!strncmp (ip->url, "file:", 5))
 			 off = 5;
 
-		 gtk_entry_set_text (GTK_ENTRY (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (data->pixmap_entry))),
+		 gtk_entry_set_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (data->pixmap_entry))),
 				     ip->url + off);
 	}
 
-	gtk_signal_connect (GTK_OBJECT (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (data->pixmap_entry))),
-			    "changed", GTK_SIGNAL_FUNC (entry_changed), data);
+	g_signal_connect (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (data->pixmap_entry)),
+			  "changed", G_CALLBACK (entry_changed), data);
 
 	gtk_box_pack_start (GTK_BOX (vbox), data->pixmap_entry, FALSE, FALSE, 0);
 
@@ -303,7 +304,7 @@ body_properties (GtkHTMLControlData *cd, gpointer *set_data)
 
 	frame = gtk_frame_new (_("Colors"));
 	vbox = gtk_vbox_new (FALSE, 2);
-	gtk_container_border_width (GTK_CONTAINER (vbox), 3);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
 
 	group = NULL;
 	i = 0;
@@ -315,9 +316,9 @@ body_properties (GtkHTMLControlData *cd, gpointer *set_data)
 				 &color->color, \
 				 color_group_fetch ("body_" g, cd)); \
         color_combo_set_color (COLOR_COMBO (combo), &data->color [ct]); \
-        gtk_object_set_data (GTK_OBJECT (combo), "type", GINT_TO_POINTER (ct)); \
-        gtk_signal_connect (GTK_OBJECT (combo), "changed", GTK_SIGNAL_FUNC (color_changed), data); \
-	hbox = gtk_hbox_new (FALSE, 3); \
+        g_object_set_data (G_OBJECT (combo), "type", GINT_TO_POINTER (ct)); \
+        g_signal_connect (combo, "color_changed", G_CALLBACK (color_changed), data); \
+	hbox = gtk_hbox_new (FALSE, 12); \
 	gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0); \
 	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (x), FALSE, FALSE, 0); \
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0); \
@@ -333,17 +334,16 @@ body_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	/* gtk_idle_add (hide_preview, data); */
 
 	gtk_widget_show_all (table);
-	gnome_pixmap_entry_set_preview (GNOME_PIXMAP_ENTRY (data->pixmap_entry), FALSE);
 
 	return table;
 }
 
-void
+gboolean
 body_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 {
 	GtkHTMLEditBodyProperties *data = (GtkHTMLEditBodyProperties *) get_data;
 	gboolean redraw = FALSE;
-	gchar *fname;
+	const gchar *fname;
 
 #define APPLY_COLOR(c) \
 	if (data->color_changed [c]) { \
@@ -351,11 +351,13 @@ body_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
                 redraw = TRUE; \
         }
 
+	cd->html->engine->leftBorder = data->left_margin;
+
 	APPLY_COLOR (HTMLTextColor);
 	APPLY_COLOR (HTMLLinkColor);
 	APPLY_COLOR (HTMLBgColor);
 
-	fname = gtk_entry_get_text (GTK_ENTRY (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (data->pixmap_entry))));
+	fname = gtk_entry_get_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (data->pixmap_entry))));
 	if (fname && *fname) {
 		HTMLEngine *e = data->cd->html->engine;
 		gchar *file = g_strconcat ("file:", fname, NULL);
@@ -369,7 +371,10 @@ body_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 
 	if (redraw)
 		gtk_widget_queue_draw (GTK_WIDGET (cd->html));
-	gtk_html_set_title (data->cd->html, e_utf8_gtk_entry_get_text (GTK_ENTRY (data->entry_title)));
+	
+	gtk_html_set_title (data->cd->html, gtk_entry_get_text (GTK_ENTRY (data->entry_title)));
+
+	return TRUE;
 }
 
 void
