@@ -41,6 +41,7 @@ struct _HTMLClueFlow {
 
 	/* Indentation level for blockquote and lists.  */
 	GByteArray *levels;
+	gint indent_width;
 
 	/* list item attributes - this will be ideally moved to list item type
 	   based on HTMLClueFlow once we have real types */
@@ -71,7 +72,7 @@ void               html_clueflow_init                         (HTMLClueFlow
 							       gint                item_number,
 							       HTMLClearType       clear);
 HTMLObject        *html_clueflow_new                          (HTMLClueFlowStyle   style,
-							       GByteArray         *leves,
+							       GByteArray         *levels,
 							       HTMLListType        item_type,
 							       gint                item_number,
 							       HTMLClearType       clear);
@@ -126,6 +127,7 @@ gboolean           html_clueflow_tabs                         (HTMLClueFlow
 							       HTMLPainter        *p);
 gboolean           html_clueflow_style_equals                 (HTMLClueFlow       *cf1,
 							       HTMLClueFlow       *cf2);
+
 void               html_clueflow_set_item_color               (HTMLClueFlow       *flow,
 							       HTMLColor          *color);
 
