@@ -21,14 +21,6 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 
-   TODO:
-
-     - implement proper animation loading (now it loops thru loaded frames
-       and does not stop when loading is in progress and we are out of frames)
-     - look at gdk-pixbuf to make gdk_pixbuf_compose work (look also
-       on gk_pixbuf_render_to_drawable_alpha)
-     - take care about all the frame->action values
-
 */
 
 #include <config.h>
@@ -37,6 +29,8 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtk.h>
 
 #include "gtkhtml.h"
 #include "gtkhtml-properties.h"
@@ -61,29 +55,31 @@
 struct _HTMLImageFactory {
 	HTMLEngine *engine;
 	GHashTable *loaded_images;
+	GdkPixbuf  *missing;
+	gboolean    animate;
 };
 
 
 #define DEFAULT_SIZE 48
-#define ANIMATIONS(i) GTK_HTML_CLASS (GTK_OBJECT (i->image_ptr->factory->engine->widget)->klass)->properties->animations
 #define STRDUP_HELPER(i,j) if (i != j) {char *tmp = g_strdup (j); g_free(i); i = tmp;}
-#define HTML_IMAGE_DEFAULT_TEXT "[image]"
+
+#define DA(x)
 
 HTMLImageClass html_image_class;
 static HTMLObjectClass *parent_class = NULL;
 
-static HTMLImageAnimation *html_image_animation_new     (HTMLImage *image);
-static void                html_image_animation_destroy (HTMLImageAnimation *anim);
-static HTMLImagePointer   *html_image_pointer_new       (const char *filename, HTMLImageFactory *factory);
-static void                html_image_pointer_ref       (HTMLImagePointer *ip);
-static void                html_image_pointer_unref     (HTMLImagePointer *ip);
-static gboolean            html_image_pointer_timeout   (HTMLImagePointer *ip);
-static void                render_cur_frame             (HTMLImage *image, gint nx, gint ny, const GdkColor *highlight_color);
+static HTMLImagePointer   *html_image_pointer_new               (const char *filename, HTMLImageFactory *factory);
+static void                html_image_pointer_ref               (HTMLImagePointer *ip);
+static void                html_image_pointer_unref             (HTMLImagePointer *ip);
+static gboolean            html_image_pointer_timeout           (HTMLImagePointer *ip);
+static gint                html_image_pointer_run_animation     (HTMLImagePointer *ip);
+static void                html_image_pointer_start_animation   (HTMLImagePointer *ip);
+
+static GdkPixbuf *         html_image_factory_get_missing       (HTMLImageFactory *factory);
 
 guint
 html_image_get_actual_width (HTMLImage *image, HTMLPainter *painter)
 {
-	GdkPixbuf *pixbuf = image->image_ptr->pixbuf;
 	GdkPixbufAnimation *anim = image->image_ptr->animation;
 	gint pixel_size = painter ? html_painter_get_pixel_size (painter) : 1;
 	gint width;
@@ -95,18 +91,16 @@ html_image_get_actual_width (HTMLImage *image, HTMLPainter *painter)
 			 * image->specified_width) / 100;
 	} else if (image->specified_width > 0) {
 		width = image->specified_width * pixel_size;
-	} else if (image->image_ptr == NULL || pixbuf == NULL) {
+	} else if (image->image_ptr == NULL || anim == NULL) {
 		width = DEFAULT_SIZE * pixel_size;
 	} else {
-		width = (((anim) ? gdk_pixbuf_animation_get_width (anim) : gdk_pixbuf_get_width (pixbuf))
-			  * pixel_size);
+		width = gdk_pixbuf_animation_get_width (anim) * pixel_size;
 
 		if (image->specified_height > 0 || image->percent_height) {
 			double scale;
 
 			scale =  ((double) html_image_get_actual_height (image, painter)) 
-				/ (((anim) ? gdk_pixbuf_animation_get_height (anim) : gdk_pixbuf_get_height (pixbuf))
-				   * pixel_size);
+				/ (gdk_pixbuf_animation_get_height (anim) * pixel_size);
 			
 			width *= scale;
 		}
@@ -119,7 +113,6 @@ html_image_get_actual_width (HTMLImage *image, HTMLPainter *painter)
 guint
 html_image_get_actual_height (HTMLImage *image, HTMLPainter *painter)
 {
-	GdkPixbuf *pixbuf = image->image_ptr->pixbuf;
 	GdkPixbufAnimation *anim = image->image_ptr->animation;
 	gint pixel_size = painter ? html_painter_get_pixel_size (painter) : 1;
 	gint height;
@@ -131,18 +124,16 @@ html_image_get_actual_height (HTMLImage *image, HTMLPainter *painter)
 			  * image->specified_height) / 100;
 	} else if (image->specified_height > 0) {
 		height = image->specified_height * pixel_size;
-	} else if (image->image_ptr == NULL || pixbuf == NULL) {
+	} else if (image->image_ptr == NULL || anim == NULL) {
 		height = DEFAULT_SIZE * pixel_size;
 	} else {
-		height = (((anim) ? gdk_pixbuf_animation_get_height (anim) : gdk_pixbuf_get_height (pixbuf))
-			  * pixel_size);
+		height = gdk_pixbuf_animation_get_height (anim) * pixel_size;
 
 		if (image->specified_width > 0 || image->percent_width) {
 			double scale;
 			
 			scale = ((double) html_image_get_actual_width (image, painter))
-				/ (((anim) ? gdk_pixbuf_animation_get_width (anim) : gdk_pixbuf_get_width (pixbuf))
-				   * pixel_size);
+				/ (gdk_pixbuf_animation_get_width (anim) * pixel_size);
 			
 			height *= scale;
 		} 
@@ -166,9 +157,6 @@ destroy (HTMLObject *o)
 	html_image_factory_unregister (image->image_ptr->factory,
 				       image->image_ptr, HTML_IMAGE (image));
 
-	if (image->animation)
-		html_image_animation_destroy (image->animation);
-
 	g_free (image->url);
 	g_free (image->target);
 	g_free (image->alt);
@@ -193,7 +181,6 @@ copy (HTMLObject *self,
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
 
 	dimg->image_ptr = simg->image_ptr;
-	dimg->animation = NULL;          /* don't bother with animation copying now. TODO */
 	dimg->color = simg->color;
 	if (simg->color)
 		html_color_ref (dimg->color);
@@ -328,24 +315,14 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 	pixel_size = html_painter_get_pixel_size (painter);
 
 	if (o->parent && HTML_IS_CLUEFLOW (o->parent)
-	    && HTML_IS_PLAIN_PAINTER (painter)) {
-		char *text = HTML_IMAGE_DEFAULT_TEXT;
+	    && HTML_IS_PLAIN_PAINTER (painter) && image->alt && *image->alt) {
 		GtkHTMLFontStyle style;
 		gint lo = 0;
 
-		if (image->alt)
-			text = image->alt;
-
 		style = html_clueflow_get_default_font_style (HTML_CLUEFLOW (o->parent));
-		
-		if (*text) {
-			o->width = html_painter_calc_text_width (painter, text, g_utf8_strlen (text, -1), &lo,
-								 style, NULL);
-		} else {
-			o->width = 0;
-		}
-		o->ascent = html_painter_calc_ascent (painter, style, NULL);
-		o->descent = html_painter_calc_descent (painter, style, NULL);
+		/* FIXME: cache items and glyphs? */
+		html_painter_calc_text_size (painter, image->alt, g_utf8_strlen (image->alt, -1), NULL, NULL, 0, &lo,
+					     style, NULL, &o->width, &o->ascent, &o->descent);
 	} else {
 		width = html_image_get_actual_width (image, painter);
 		height = html_image_get_actual_height (image, painter);
@@ -367,14 +344,20 @@ static void
 draw_plain (HTMLObject *o, HTMLPainter *p, gint x, gint y, gint width, gint height, gint tx, gint ty)
 {
 	HTMLImage *img = HTML_IMAGE (o);
-	char *text = HTML_IMAGE_DEFAULT_TEXT;
 
-	if (img->alt)
-		text = img->alt;
+	if (img->alt && *img->alt) {
 
-	if (*text) {
-		html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLTextColor)->color);
-		html_painter_draw_text (p, o->x + tx, o->y + ty, text, g_utf8_strlen (text, -1), 0);
+		/* FIXME: cache items and glyphs? */
+		if (o->selected) {
+			html_painter_set_pen (p, &html_colorset_get_color_allocated
+					      (p, p->focus ? HTMLHighlightColor : HTMLHighlightNFColor)->color);
+			html_painter_fill_rect (p, o->x + tx, o->y + ty - o->ascent, o->width, o->ascent + o->descent);
+			html_painter_set_pen (p, &html_colorset_get_color_allocated
+					      (p, p->focus ? HTMLHighlightTextColor : HTMLHighlightTextNFColor)->color);
+		} else { 
+			html_painter_set_pen (p, &html_colorset_get_color_allocated (p, HTMLTextColor)->color);
+		}
+		html_painter_draw_text (p, o->x + tx, o->y + ty, img->alt, g_utf8_strlen (img->alt, -1), NULL, NULL, 0, 0);
 	}
 }
 
@@ -386,17 +369,17 @@ draw (HTMLObject *o,
       gint tx, gint ty)
 {
 	HTMLImage *image;
+	HTMLImagePointer *ip;
 	GdkPixbuf *pixbuf;
 	gint base_x, base_y;
 	gint scale_width, scale_height;
 	GdkColor *highlight_color;
 	guint pixel_size;
-	ArtIRect paint;
+	GdkRectangle paint;
 
 	/* printf ("Image::draw\n"); */
 
-	html_object_calc_intersection (o, &paint, x, y, width, height);
-	if (art_irect_empty (&paint))
+	if (!html_object_intersect (o, &paint, x, y, width, height))
 		return;
 
 	if (HTML_IS_PLAIN_PAINTER (painter)) {
@@ -405,8 +388,18 @@ draw (HTMLObject *o,
 	}
 
 	image = HTML_IMAGE (o);
+	ip = image->image_ptr;
+
+	if (ip->animation) {
+		if (HTML_IS_GDK_PAINTER (painter) && !gdk_pixbuf_animation_is_static_image (ip->animation)) {
+			pixbuf = gdk_pixbuf_animation_iter_get_pixbuf (ip->iter);
+		} else {
+			pixbuf = gdk_pixbuf_animation_get_static_image (ip->animation);
+		}
+	} else {
+		pixbuf = NULL;
+	}
 
-	pixbuf = image->image_ptr->pixbuf;
 	pixel_size = html_painter_get_pixel_size (painter);
 
 	if (o->selected) {
@@ -415,6 +408,9 @@ draw (HTMLObject *o,
 	} else
 		highlight_color = NULL;
 
+	base_x = o->x + tx + (image->border + image->hspace) * pixel_size;
+	base_y = o->y + ty + (image->border + image->vspace) * pixel_size - o->ascent;
+
 	if (pixbuf == NULL) {
 		gint vspace, hspace;
 
@@ -439,10 +435,22 @@ draw (HTMLObject *o,
 					 o->width - 2 * hspace,
 					 o->ascent + o->descent - 2 * vspace,
 					 GTK_HTML_ETCH_IN, 1);
+
+		if (ip->factory)
+			pixbuf = html_image_factory_get_missing (ip->factory);
+
+		if (pixbuf && 
+		    (o->width > gdk_pixbuf_get_width (pixbuf)) &&
+		    (o->ascent  + o->descent > gdk_pixbuf_get_height (pixbuf)))
+			html_painter_draw_pixmap (painter, pixbuf,
+						  base_x, base_y,
+						  gdk_pixbuf_get_width (pixbuf) * pixel_size,
+						  gdk_pixbuf_get_height (pixbuf) * pixel_size,
+						  highlight_color);
+			
+			
 		return;
 	}
-	base_x = o->x + tx + (image->border + image->hspace) * pixel_size;
-	base_y = o->y + ty + (image->border + image->vspace) * pixel_size - o->ascent;
 
 	scale_width = html_image_get_actual_width (image, painter);
 	scale_height = html_image_get_actual_height (image, painter);
@@ -462,20 +470,12 @@ draw (HTMLObject *o,
 					 GTK_HTML_ETCH_NONE, image->border);
 		
 	}
-	if (ANIMATIONS (image) &&  image->animation && ! HTML_IS_PRINTER (painter)) {
-		image->animation->active = TRUE;
-		image->animation->x = base_x;
-		image->animation->y = base_y;
-		image->animation->ex = image->image_ptr->factory->engine->x_offset;
-		image->animation->ey = image->image_ptr->factory->engine->y_offset;
-
-		render_cur_frame (image, base_x, base_y, highlight_color);
-	} else {
-		html_painter_draw_pixmap (painter, pixbuf,
-					  base_x, base_y,
-					  scale_width, scale_height,
-					  highlight_color);
-	}
+	
+	image->animation_active = TRUE;
+	html_painter_draw_pixmap (painter, pixbuf,
+				  base_x, base_y,
+				  scale_width, scale_height,
+				  highlight_color);
 }
 
 gchar *
@@ -485,19 +485,22 @@ html_image_resolve_image_url (GtkHTML *html, gchar *image_url)
 
 	/* printf ("html_image_resolve_image_url %p\n", html->editor_api); */
 	if (html->editor_api) {
-		GtkArg *args [2];
+		GValue  *iarg = g_new0 (GValue, 1);
+		GValue  *oarg;
 
-		args [0] = gtk_arg_new (GTK_TYPE_STRING);
-		GTK_VALUE_STRING (*args [0]) = image_url;
+		g_value_init (iarg, G_TYPE_STRING);
+		g_value_set_string (iarg, image_url);
 
-		args [1] = (* html->editor_api->event) (html, GTK_HTML_EDITOR_EVENT_IMAGE_URL, args, html->editor_data);
-		gtk_arg_free (args [0], FALSE);
+		oarg = (* html->editor_api->event) (html, GTK_HTML_EDITOR_EVENT_IMAGE_URL, iarg, html->editor_data);
 
-		if (args [1]) {
-			if (args [1]->type == GTK_TYPE_STRING)
-				url = GTK_VALUE_STRING (*args [1]);
-			gtk_arg_free (args [1], FALSE);
+		if (oarg) {
+			if (G_VALUE_TYPE (oarg) == G_TYPE_STRING)
+				url = (gchar *) g_strdup (g_value_get_string (oarg));
+			g_value_unset (oarg);	
+			g_free (oarg);
 		}
+		g_value_unset (iarg);
+		g_free (iarg);
 	}
 	if (!url)
 		url = g_strdup (image_url);
@@ -598,16 +601,11 @@ save_plain (HTMLObject *self,
 {
 	HTMLImage *image;
 	gboolean rv = TRUE;
-	char *text = HTML_IMAGE_DEFAULT_TEXT;
 
 	image = HTML_IMAGE (self);
 	
-	if (image->alt) {
-	        text = image->alt;
-	} 
-
-	if (*text)
-		rv = html_engine_save_output_string (state, "%s", text);
+	if (image->alt)
+		rv = html_engine_save_output_string (state, "%s", image->alt);
 
 	return rv;
 }
@@ -774,7 +772,6 @@ html_image_init (HTMLImage *image,
 		image->have_color = FALSE;
 	}
 
-	image->animation = NULL;
 	image->alt = NULL;
 
 	image->hspace = 0;
@@ -937,16 +934,50 @@ html_image_set_size (HTMLImage *image, gint w, gint h, gboolean pw, gboolean ph)
 	}
 }
 
-char *image_content_types[] = {"image/*", NULL};
+static char *fallback_image_content_types[] = {"image/*", NULL};
 
 static char **
 html_image_factory_types (GtkHTMLStream *stream,
 			  gpointer user_data)
 {
-	/* FIXME: this should use the not currently existant function
-	 * in gdk-pixbuf that tells us what mime types are supported 
-	 * by the modules it has loaded.
-	 */
+	static char**image_content_types = NULL;
+	
+#if 0
+	/* this code should work in gtk+-2.2 but it is untested */
+	if (image_content_types == NULL) {
+		GSList *formats;
+		GSList *cur;
+		GSList *types = NULL;
+		gint i;
+
+		formats = gdk_pixbuf_get_formats ();
+		
+		for (cur = formats; cur; cur = cur->next) {
+			GdkPixbufFormat *format = cur->data;
+			char **mime;
+
+			mime = gdk_pixbuf_formats_get_mime_types ();
+			for (i = 0; mime && mime[i]; i++)
+				g_slist_prepend (types, g_strdup (mime[i]));
+
+		}
+		g_slist_free (formats);
+
+		if (types) {
+			image_content_types = g_new0 (char *, g_slist_length (types) + 1);
+			
+			for (cur = types, i = 0; cur; cur = cur->next, i++) {
+				image_content_types[i] = cur->data;
+			}
+			g_slist_free (types);
+		} else {
+			image_content_types = fallback_image_content_types;
+		}
+	}
+#else 
+	image_content_types = fallback_image_content_types;
+#endif
+
 	return image_content_types;
 }
 
@@ -1003,13 +1034,18 @@ html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 {
 	HTMLImagePointer *ip = user_data;
 
-	gdk_pixbuf_loader_close (ip->loader);
-	if (!ip->animation && !ip->pixbuf) {
-		ip->pixbuf = gdk_pixbuf_loader_get_pixbuf (ip->loader);
+	gdk_pixbuf_loader_close (ip->loader, NULL);
 
-		if (ip->pixbuf)
-			gdk_pixbuf_ref (ip->pixbuf);
+	if (!ip->animation) {
+		ip->animation = gdk_pixbuf_loader_get_animation (ip->loader);
+
+		if (ip->animation)
+			g_object_ref (ip->animation);
 	}
+	html_image_pointer_start_animation (ip);
+
+	g_object_unref (ip->loader);
+	ip->loader = NULL;
 
 	update_or_redraw (ip);
 	if (ip->factory->engine->opened_streams)
@@ -1029,7 +1065,7 @@ html_image_factory_write_pixbuf (GtkHTMLStream *stream,
 	HTMLImagePointer *p = user_data;
 
 	/* FIXME ! Check return value */
-	gdk_pixbuf_loader_write (p->loader, buffer, size);
+	gdk_pixbuf_loader_write (p->loader, buffer, size, NULL);
 }
 
 static void
@@ -1042,179 +1078,81 @@ static void
 html_image_factory_area_prepared (GdkPixbufLoader *loader, HTMLImagePointer *ip)
 {
 	if (!ip->animation) {
-		ip->pixbuf = gdk_pixbuf_loader_get_pixbuf (ip->loader);
-		g_assert (ip->pixbuf);
-
-		gdk_pixbuf_ref (ip->pixbuf);
-		update_or_redraw (ip);
+		ip->animation = gdk_pixbuf_loader_get_animation (loader);
+		g_object_ref (ip->animation);
+		
+		html_image_pointer_start_animation (ip);
 	}
+	update_or_redraw (ip);
 }
 
 static void
-render_cur_frame (HTMLImage *image, gint nx, gint ny, const GdkColor *highlight_color)
+html_image_pointer_queue_animation (HTMLImagePointer *ip)
 {
-	GdkPixbufFrame    *frame;
-	HTMLPainter       *painter;
-	HTMLImageAnimation *anim = image->animation;
-	GdkPixbufAnimation *ganim = image->image_ptr->animation;
-	GList *cur = gdk_pixbuf_animation_get_frames (ganim);
-	gint w, h;
-	gboolean saved_alpha = TRUE;
-
-	painter = image->image_ptr->factory->engine->painter;
-
-	frame = (GdkPixbufFrame *) anim->cur_frame->data;
-	/* printf ("w: %d h: %d action: %d\n", w, h, frame->action); */
-
-	/* FIXME this is hack to turn off alpha blending while rending 
-	 * animations.  This breaks nothing since gdk-pixbuf doesn't support animations
-	 * for anything but gif and it makes a huge difference if there are lots of
-	 * frames.  We really should add a parameter to the html_painter_draw_pixmap
-	 * call and track wether each image has full alpha or bilevel alpha, but that
-	 * will wait for a bit.
-	 */  
-	if (HTML_IS_GDK_PAINTER (painter)) {
-		saved_alpha = HTML_GDK_PAINTER (painter)->alpha;
-		HTML_GDK_PAINTER (painter)->alpha = FALSE;
-	}
-
-	do {
-		frame = (GdkPixbufFrame *) cur->data;
-		w = gdk_pixbuf_get_width (gdk_pixbuf_frame_get_pixbuf (frame));
-		h = gdk_pixbuf_get_height (gdk_pixbuf_frame_get_pixbuf (frame));
-		if (anim->cur_frame == cur) {
-			html_painter_draw_pixmap (painter, gdk_pixbuf_frame_get_pixbuf (frame),
-						  nx + gdk_pixbuf_frame_get_x_offset (frame),
-						  ny + gdk_pixbuf_frame_get_y_offset (frame),
-						  w, h,
-						  highlight_color);
-			break;
-		} else if (gdk_pixbuf_frame_get_action (frame) == GDK_PIXBUF_FRAME_RETAIN) {
-			html_painter_draw_pixmap (painter, gdk_pixbuf_frame_get_pixbuf (frame),
-						  nx + gdk_pixbuf_frame_get_x_offset (frame),
-						  ny + gdk_pixbuf_frame_get_y_offset (frame),
-						  w, h,
-						  highlight_color);
-		} 
-		cur = cur->next;
-	} while (1);
-
-	if (HTML_IS_GDK_PAINTER (painter)) {
-		HTML_GDK_PAINTER (painter)->alpha = saved_alpha;
-	}
+	gint delay = gdk_pixbuf_animation_iter_get_delay_time (ip->iter);
 
+	if (delay >= 0 && !ip->animation_timeout && ip->factory && ip->factory->animate) {
+		ip->animation_timeout = g_timeout_add (delay, 
+						       (GtkFunction) html_image_pointer_run_animation, 
+						       (gpointer) ip);
+	}	
 }
 
 static gint
-html_image_animation_timeout (HTMLImage *image)
+html_image_pointer_run_animation (HTMLImagePointer *ip)
 {
-	HTMLImageAnimation *anim = image->animation;
-	GdkPixbufAnimation *ganim = image->image_ptr->animation;
-	GdkPixbufFrame    *frame;
-	HTMLEngine        *engine;
-	gint nx, ny, nex, ney;
+	GdkPixbufAnimationIter  *iter = ip->iter;
+	HTMLEngine              *engine = ip->factory->engine;
+	
+	g_return_val_if_fail (ip->factory != NULL, FALSE);
+	ip->animation_timeout = 0;
 
 	/* printf ("animation_timeout\n"); */
-	anim->cur_frame = anim->cur_frame->next;
-	if (!anim->cur_frame)
-		anim->cur_frame = gdk_pixbuf_animation_get_frames (image->image_ptr->animation);
+	if (gdk_pixbuf_animation_iter_advance (iter, NULL)) {
+		GSList *cur;
 
-	frame = (GdkPixbufFrame *) anim->cur_frame->data;
-	
-	/* draw only if animation is active - onscreen */
+		for (cur = ip->interests; cur; cur = cur->next) {
+			HTMLImage           *image = cur->data;
 
-	engine = image->image_ptr->factory->engine;
-
-	nex = engine->x_offset;
-	ney = engine->y_offset;
-
-	nx = anim->x - (nex - anim->ex);
-	ny = anim->y - (ney - anim->ey);
-	
-	if (anim->active) {
-		gint aw, ah;
-
-		aw = gdk_pixbuf_animation_get_width (ganim);
-		ah = gdk_pixbuf_animation_get_height (ganim);
-
-		if (MAX(0, nx) < MIN(engine->width, nx+aw)
-		    && MAX(0, ny) < MIN(engine->height, ny+ah)) {
-			html_engine_draw (engine,
-					  nx, ny,
-					  aw, ah);
-			/* html_engine_queue_draw (engine, HTML_OBJECT (image)); */
-			
+			if (image && image->animation_active) {
+				image->animation_active = FALSE;
+				html_engine_queue_draw (engine, HTML_OBJECT (image));
+			}
 		}
-		
 	}
-	/* printf ("timeout: %d\n", gdk_pixbuf_frame_get_delay_time (frame)); */
-	anim->timeout = g_timeout_add (10 * (gdk_pixbuf_frame_get_delay_time (frame) > 0
-					     ? gdk_pixbuf_frame_get_delay_time (frame) : 1),
-				       (GtkFunction) html_image_animation_timeout, (gpointer) image);
-
+		
+	html_image_pointer_queue_animation (ip);
 	return FALSE;
 }
 
 static void
-html_image_animation_start (HTMLImage *image)
+html_image_pointer_start_animation (HTMLImagePointer *ip)
 {
-	HTMLImageAnimation *anim = image->animation;
+	if (ip->animation && !gdk_pixbuf_animation_is_static_image (ip->animation)) {
+		if (!ip->iter)
+			ip->iter = gdk_pixbuf_animation_get_iter (ip->animation, NULL);
 
-	if (anim && gdk_pixbuf_animation_get_num_frames (image->image_ptr->animation) > 1) {
-		if (anim->timeout == 0) {
-			GList *frames = gdk_pixbuf_animation_get_frames (image->image_ptr->animation);
-
-			anim->cur_frame = frames->next;
-			anim->cur_n = 1;
-			anim->timeout = g_timeout_add (10 * gdk_pixbuf_frame_get_delay_time
-						       ((GdkPixbufFrame *) frames->data),
-						       (GtkFunction) html_image_animation_timeout, (gpointer) image);
-		}
+		html_image_pointer_queue_animation (ip);
 	}
 }
 
 static void
-html_image_animation_stop (HTMLImageAnimation *anim)
+html_image_pointer_stop_animation (HTMLImagePointer *ip)
 {
-	if (anim->timeout) {
-		g_source_remove (anim->timeout);
-		anim->timeout = 0;
-	}
-	anim->active  = 0;
-}
-
-static void
-html_image_factory_frame_done (GdkPixbufLoader *loader, GdkPixbufFrame *frame, HTMLImagePointer *ip)
-{
-	/* printf ("frame done\n"); */
-
-	if (!ip->animation) {
-		ip->animation = gdk_pixbuf_loader_get_animation (loader);
-		gdk_pixbuf_animation_ref (ip->animation);
-	}
-	g_assert (ip->animation);
-
-	if (gdk_pixbuf_animation_get_num_frames (ip->animation) > 1) {
-		GSList *cur = ip->interests;
-		HTMLImage *image;
-
-		while (cur) {
-			if (cur->data) {
-				image = HTML_IMAGE (cur->data);
-				if (!image->animation) {
-					image->animation = html_image_animation_new (image);
-				}
-				html_image_animation_start (image);
-			}
-			cur = cur->next;
-		}
+	if (ip->animation_timeout) {
+		g_source_remove (ip->animation_timeout);
+		ip->animation_timeout = 0;
 	}
 }
 
-static void
-html_image_factory_animation_done (GdkPixbufLoader *loader, HTMLImagePointer *ip)
+static GdkPixbuf *
+html_image_factory_get_missing (HTMLImageFactory *factory)
 {
-	g_print ("animation done\n");
+	if (!factory->missing)
+		factory->missing = gtk_widget_render_icon (GTK_WIDGET (factory->engine->widget),
+							  GTK_STOCK_MISSING_IMAGE,
+							  GTK_ICON_SIZE_BUTTON, "GtkHTML.ImageMissing");
+	return factory->missing;
 }
 
 HTMLImageFactory *
@@ -1224,33 +1162,33 @@ html_image_factory_new (HTMLEngine *e)
 	retval = g_new (HTMLImageFactory, 1);
 	retval->engine = e;
 	retval->loaded_images = g_hash_table_new (g_str_hash, g_str_equal);
+	retval->missing = NULL;
+	retval->animate = TRUE;
 
 	return retval;
 }
 
 static gboolean
-cleanup_images (gpointer key, gpointer value, gpointer user_data)
+cleanup_images (gpointer key, gpointer value, gpointer free_everything)
 {
-	HTMLImagePointer *ptr;
-	gboolean retval = FALSE;
+	HTMLImagePointer *ip = value;
 
-	ptr = value;
-
-	/* user data means: NULL only clean, non-NULL free */
-	if (user_data){
-		if (ptr->interests != NULL) {
-			g_slist_free (ptr->interests);
-			ptr->interests = NULL;
+	/* free_everything means: NULL only clean, non-NULL free */
+	if (free_everything){
+		if (ip->interests != NULL) {
+			g_slist_free (ip->interests);
+			ip->interests = NULL;
 		}
 	}
 
 	/* clean only if this image is not used anymore */
-	if (!ptr->interests){
-		retval = TRUE;
-		html_image_pointer_unref (ptr);
+	if (!ip->interests){
+		html_image_pointer_unref (ip);
+		ip->factory = NULL;
+		return TRUE;
 	}
 
-	return retval;
+	return FALSE;
 }
 
 void
@@ -1267,38 +1205,13 @@ html_image_factory_free (HTMLImageFactory *factory)
 
 	g_hash_table_foreach_remove (factory->loaded_images, cleanup_images, factory);
 	g_hash_table_destroy (factory->loaded_images);
+
+	if (factory->missing)
+		gdk_pixbuf_unref (factory->missing);
+
 	g_free (factory);
 }
 
-static HTMLImageAnimation *
-html_image_animation_new (HTMLImage *image)
-{
-	HTMLImageAnimation *animation;
-
-	animation = g_new (HTMLImageAnimation, 1);
-	animation->cur_frame = gdk_pixbuf_animation_get_frames (image->image_ptr->animation);
-	animation->cur_n = 0;
-	animation->x = 0;
-	animation->y = 0;
-	animation->ex = 0;
-	animation->ey = 0;
-	animation->timeout = 0;
-	animation->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-					    gdk_pixbuf_animation_get_width (image->image_ptr->animation),
-					    gdk_pixbuf_animation_get_height (image->image_ptr->animation));
-	animation->active = FALSE;
-
-	return animation;
-}
-
-static void
-html_image_animation_destroy (HTMLImageAnimation *anim)
-{
-	html_image_animation_stop (anim);
-	gdk_pixbuf_unref (anim->pixbuf);
-	g_free (anim);
-}
-
 #define STALL_INTERVAL 1000
 
 static HTMLImagePointer *
@@ -1310,7 +1223,7 @@ html_image_pointer_new (const char *filename, HTMLImageFactory *factory)
 	retval->refcount = 1;
 	retval->url = g_strdup (filename);
 	retval->loader = gdk_pixbuf_loader_new ();
-	retval->pixbuf = NULL;
+	retval->iter = NULL;
 	retval->animation = NULL;
 	retval->interests = NULL;
 	retval->factory = factory;
@@ -1318,6 +1231,7 @@ html_image_pointer_new (const char *filename, HTMLImageFactory *factory)
 	retval->stall_timeout = gtk_timeout_add (STALL_INTERVAL, 
 						 (GtkFunction)html_image_pointer_timeout,
 						 retval);
+	retval->animation_timeout = 0;
 	return retval;
 }
 
@@ -1327,6 +1241,10 @@ html_image_pointer_timeout (HTMLImagePointer *ip)
 	GSList *list;
 	HTMLImage *image;
 
+	ip->stall_timeout = 0;
+
+	g_return_val_if_fail (ip->factory != NULL, FALSE);
+
 	ip->stall = TRUE;
 
 	list = ip->interests;
@@ -1334,7 +1252,7 @@ html_image_pointer_timeout (HTMLImagePointer *ip)
 	 * draw the frame now that we decided they've had enough time to
 	 * load the image
 	 */
-	if (ip->pixbuf == NULL) {
+	if (ip->animation == NULL) {
 		while (list) {
 			image = (HTMLImage *)list->data;
 
@@ -1345,7 +1263,6 @@ html_image_pointer_timeout (HTMLImagePointer *ip)
 			list = list->next;
 		}
 	}
-	ip->stall_timeout = 0;
 	return FALSE;
 }
 
@@ -1359,18 +1276,18 @@ static void
 free_image_ptr_data (HTMLImagePointer *ip)
 {
 	if (ip->loader) {
-		gtk_object_unref (GTK_OBJECT (ip->loader));
+		gdk_pixbuf_loader_close (ip->loader, NULL);
+		g_object_unref (ip->loader);
 		ip->loader = NULL;
 	}
 	if (ip->animation) {
-		gdk_pixbuf_animation_unref (ip->animation);
+		g_object_unref (ip->animation);
 		ip->animation = NULL;
 	}
-	if (ip->pixbuf) {
-		gdk_pixbuf_unref (ip->pixbuf);
-		ip->pixbuf = NULL;
+	if (ip->iter) {
+		g_object_unref (ip->iter);
+		ip->iter = NULL;
 	}
-
 }
 
 static void
@@ -1389,90 +1306,74 @@ html_image_pointer_unref (HTMLImagePointer *ip)
 
 	ip->refcount--;
 	/* printf ("unref(%p) %s --> %d\n", ip, ip->url, ip->refcount); */
-	if (ip->refcount <= 0) {
+	if (ip->refcount < 1) {
 		/* printf ("freeing %s\n", ip->url); */
 		html_image_pointer_remove_stall (ip);
+		html_image_pointer_stop_animation (ip);
 		g_free (ip->url);
 		free_image_ptr_data (ip);
 		g_free (ip);
 	}
 }
 
-static void
+GtkHTMLStream *
 html_image_pointer_load (HTMLImagePointer *ip)
 {
-	GtkHTMLStream *handle;
-
 	html_image_pointer_ref (ip);
 
-	handle = gtk_html_stream_new (GTK_HTML (ip->factory->engine->widget),
-				      html_image_factory_types,
-				      html_image_factory_write_pixbuf,
-				      html_image_factory_end_pixbuf,
-				      ip);
 	ip->factory->engine->opened_streams ++;
-
-	/* This is a bit evil, I think.  But it's a lot better here
-	   than in the HTMLImage object.  FIXME anyway -- ettore  */
-	/* printf ("url_requested(%p) %s\n", ip->factory->engine->widget, ip->url); */
-	gtk_signal_emit_by_name (GTK_OBJECT (ip->factory->engine), "url_requested", ip->url, handle);
+	return gtk_html_stream_new (GTK_HTML (ip->factory->engine->widget),
+				    html_image_factory_types,
+				    html_image_factory_write_pixbuf,
+				    html_image_factory_end_pixbuf,
+				    ip);
 }
 
 HTMLImagePointer *
-html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char *filename, gboolean reload)
+html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char *url, gboolean reload)
 {
-	HTMLImagePointer *retval;
+	HTMLImagePointer *ip;
+	GtkHTMLStream *stream = NULL;
 
 	g_return_val_if_fail (factory, NULL);
-	g_return_val_if_fail (filename, NULL);
+	g_return_val_if_fail (url, NULL);
 
-	retval = g_hash_table_lookup (factory->loaded_images, filename);
+	ip = g_hash_table_lookup (factory->loaded_images, url);
 
-	if (!retval){
-		retval = html_image_pointer_new (filename, factory);
-		if (*filename) {
-			gtk_signal_connect (GTK_OBJECT (retval->loader), "area_prepared",
-					    GTK_SIGNAL_FUNC (html_image_factory_area_prepared),
-					    retval);
+	if (!ip) {
+		ip = html_image_pointer_new (url, factory);
+		g_hash_table_insert (factory->loaded_images, ip->url, ip);
+		if (*url) {
+			g_signal_connect (G_OBJECT (ip->loader), "area_prepared",
+					  G_CALLBACK (html_image_factory_area_prepared),
+					  ip);
 
-			gtk_signal_connect (GTK_OBJECT (retval->loader), "area_updated",
-					    GTK_SIGNAL_FUNC (html_image_factory_area_updated),
-					    retval);
-
-			gtk_signal_connect (GTK_OBJECT (retval->loader), "frame_done",
-					    GTK_SIGNAL_FUNC (html_image_factory_frame_done),
-					    retval);
-
-			gtk_signal_connect (GTK_OBJECT (retval->loader), "animation_done",
-					    GTK_SIGNAL_FUNC (html_image_factory_animation_done),
-					    retval);
-
-			g_hash_table_insert (factory->loaded_images, retval->url, retval);
-			html_image_pointer_load (retval);
+			g_signal_connect (G_OBJECT (ip->loader), "area_updated",
+					  G_CALLBACK (html_image_factory_area_updated),
+					  ip);
+			stream = html_image_pointer_load (ip);
 		}
 	} else {
 		if (reload) {
-			free_image_ptr_data (retval);
-			retval->loader = gdk_pixbuf_loader_new ();
-			html_image_pointer_load (retval);
+			free_image_ptr_data (ip);
+			ip->loader = gdk_pixbuf_loader_new ();
+			stream = html_image_pointer_load (ip);
 		}
 	}
 
-	html_image_pointer_ref (retval);
+	if (stream)
+		g_signal_emit_by_name (factory->engine, "url_requested", ip->url, stream);
+
+	html_image_pointer_ref (ip);
 
 	/* we add also NULL ptrs, as we dont want these to be cleaned out */
-	retval->interests = g_slist_prepend (retval->interests, i);
+	ip->interests = g_slist_prepend (ip->interests, i);
 
 	if (i) {
-		i->image_ptr      = retval;
-
-		if (retval->animation && gdk_pixbuf_animation_get_num_frames (retval->animation) > 1) {
-			i->animation = html_image_animation_new (i);
-			html_image_animation_start (i);
-		}
+		i->image_ptr = ip;
 	}
 
-	return retval;
+	return ip;
 }
 
 #if 0
@@ -1489,10 +1390,20 @@ html_image_factory_unregister (HTMLImageFactory *factory, HTMLImagePointer *poin
 	pointer->interests = g_slist_remove (pointer->interests, i);
 
 	html_image_pointer_unref (pointer);
+
 	if (pointer->refcount == 1) {
 		g_assert (pointer->interests == NULL);
-		/* printf ("remove %s\n", pointer->url); */
-		g_hash_table_remove (factory->loaded_images, pointer->url);
+		/*
+		 * FIXME The factory can be NULL if the image was from an iframe 
+		 * that has been destroyed and the image is living in the cut buffer
+		 * this isn't particularly clean and should be refactored.
+		 *
+		 * We really need a way to let cut objects know they are living outside
+		 * the normal flow.
+		 */
+		if (factory) 
+			g_hash_table_remove (factory->loaded_images, pointer->url);
+		pointer->factory = NULL;
 		html_image_pointer_unref (pointer);
 	}
 }
@@ -1501,28 +1412,50 @@ static void
 stop_anim (gpointer key, gpointer value, gpointer user_data)
 {
 	HTMLImagePointer *ip = value;
-	GSList *cur = ip->interests;
-	HTMLImage *image;
-
 	html_image_pointer_remove_stall (ip);
-
-	while (cur) {
-		if (cur->data) {
-			image = (HTMLImage *) cur->data;
-			if (image->animation) {
-				html_image_animation_stop (image->animation);
-			}
-		}
-		cur = cur->next;
-	}
+	html_image_pointer_stop_animation (ip);
 }
 
 void
 html_image_factory_stop_animations (HTMLImageFactory *factory)
 {
+	DA (g_warning ("stop animations");)
 	g_hash_table_foreach (factory->loaded_images, stop_anim, NULL);
 }
 
+static void
+start_anim (gpointer key, gpointer value, gpointer user_data)
+{
+	HTMLImagePointer *ip = value;
+	html_image_pointer_start_animation (ip);
+}
+
+void
+html_image_factory_start_animations (HTMLImageFactory *factory)
+{
+	DA (g_warning ("start animations");)
+	g_hash_table_foreach (factory->loaded_images, start_anim, NULL);
+}
+
+gboolean
+html_image_factory_get_animate (HTMLImageFactory *factory)
+{
+	return factory->animate;
+}
+
+void
+html_image_factory_set_animate (HTMLImageFactory *factory, gboolean animate)
+{
+	if (animate != factory->animate) {
+		factory->animate = animate;
+
+		if (animate)
+			html_image_factory_start_animations (factory);
+		else 
+			html_image_factory_stop_animations (factory);
+	}
+}
+
 static gboolean
 move_image_pointers (gpointer key, gpointer value, gpointer data)
 {
@@ -1530,8 +1463,9 @@ move_image_pointers (gpointer key, gpointer value, gpointer data)
 	HTMLImagePointer *ip  = HTML_IMAGE_POINTER (value);
 
 	ip->factory = dst;
+	
 	g_hash_table_insert (dst->loaded_images, ip->url, ip);
-	html_image_pointer_load (ip);
+	g_signal_emit_by_name (ip->factory->engine, "url_requested", ip->url, html_image_pointer_load (ip));
 
 	return TRUE;
 }
@@ -1548,13 +1482,12 @@ deactivate_anim (gpointer key, gpointer value, gpointer user_data)
 	HTMLImagePointer *ip = value;
 	GSList *cur = ip->interests;
 	HTMLImage *image;
+	
 
 	while (cur) {
 		if (cur->data) {
 			image = (HTMLImage *) cur->data;
-			if (image->animation) {
-				image->animation->active = 0;
-			}
+			image->animation_active = FALSE;
 		}
 		cur = cur->next;
 	}
